# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)
from werkzeug.exceptions import NotFound

from odoo import SUPERUSER_ID, _, http
from odoo.exceptions import AccessError, MissingError
from odoo.http import Response, request
from odoo.osv import expression

from odoo.addons.account.controllers.portal import PortalAccount as PortalAccount
from odoo.addons.http_routing.models.ir_http import slug


class PortalAccount(PortalAccount):
    def _invoice_get_page_view_values(self, invoice, access_token, **kwargs):
        """
        Override default function to configure page_name
        """
        values = super()._invoice_get_page_view_values(invoice, access_token, **kwargs)
        values.update(self._get_role(invoice.acc_operation_id))
        values["page_name"] = (
            "Facture " + invoice.name
            if invoice.state != "draft"
            else "Facture Brouillon"
        )
        return values

    def _prepare_invoice_portal_values(self, operation, domain, order="name desc"):
        """
        Get all invoices matching domain
        """
        values = {}
        domain = domain if domain else []
        values.update(self._get_role(operation))
        domain = expression.AND(
            [
                domain,
                [("acc_operation_id", "=", operation.id)],
                [("acc_delivery_counter_id", "!=", False)],
                [
                    (
                        "move_type",
                        "in",
                        (
                            "out_invoice",
                            "out_refund",
                            "in_invoice",
                            "in_refund",
                            "out_receipt",
                            "in_receipt",
                        ),
                    )
                ],
            ]
        )
        # Si pas PMO, admin ou SuperAdmin, il faut limiter plus les factures retournées
        if (
            not values.get("isSuperAdmin")
            and not values.get("isAdmin")
            and not values.get("isPmo")
        ):
            subdomain_prod = [
                (
                    "company_id.partner_id",
                    "=",
                    request.env.user.commercial_partner_id.id,
                )
            ]
            subdomain_cons = expression.AND(
                [
                    [("state", "not in", ("cancel", "draft"))],
                    [
                        (
                            "partner_id.commercial_partner_id",
                            "=",
                            request.env.user.commercial_partner_id.id,
                        )
                    ],
                ]
            )
            if values.get("isProductor") and not values.get("isConsumer"):
                # Si producteur, alors on affiche toutes les factures de la société
                domain = expression.AND(
                    [
                        domain,
                        subdomain_prod,
                    ]
                )
            elif values.get("isProductor") and values.get("isConsumer"):
                # Si producteur et consommateur, afficher les 2 règles avec un OU
                subdomain = expression.OR([subdomain_prod, subdomain_cons])
                domain = expression.AND(
                    [
                        domain,
                        subdomain,
                    ]
                )
            else:
                # Si uniquement consommateur (ou aucun droit) n'afficher que
                # les factures validées assignées à ce partenaire
                domain = expression.AND(
                    [
                        domain,
                        subdomain_cons,
                    ]
                )

        values["invoices"] = (
            request.env["account.move"].sudo().search(domain, order=order)
        )
        return values

    def _prepare_invoice_document_portal_values(self, operation, order):
        """
        Get all invoices document matching domain
        """
        values = {}
        values.update(self._get_role(operation))
        domain = expression.AND(
            [
                [("type", "=", "facture")],
                [("acc_operation_id", "=", operation.id)],
            ]
        )

        if (
            not values.get("isPmo")
            and not values.get("isProductor")
            and not values.get("isAdmin")
            and not values.get("isSuperAdmin")
        ):
            domain = expression.AND(
                [
                    domain,
                    [
                        (
                            "seller_id.commercial_partner_id",
                            "=",
                            request.env.user.commercial_partner_id.id,
                        )
                    ],
                ]
            )

        values["contracts"] = (
            request.env["acc.contract"].sudo().search(domain, order=order)
        )
        return values

    @http.route(
        ["/invoice/<int:invoice_id>"],
        type="http",
        auth="public",
        website=True,
        csrf=False,
        sitemap=False,
    )
    def print_invoice(self, invoice_id, **kw):
        """
        Route to print given invoice after checking access rights
        """
        invoice = request.env["account.move"].browse([invoice_id])
        invoice_sudo = invoice.with_user(SUPERUSER_ID).exists()
        if not invoice_sudo:
            return Response(
                _("Aucun document n'est disponible à cette URL."), status=404
            )
        else:
            return self._print_pdf(invoice_sudo)

    @http.route()
    def portal_my_invoice_detail(
        self, invoice_id, access_token=None, report_type=None, download=False, **kw
    ):
        """
        Overrides account portal default method to use specific template if
        invoice is an OACC one
        """
        try:
            invoice_sudo = self._document_check_access(
                "account.move", invoice_id, access_token
            )
        except (AccessError, MissingError) as e:
            raise NotFound() from e

        if (
            invoice_sudo.acc_operation_id
            and invoice_sudo.acc_delivery_counter_id
            and report_type in ("html", "pdf", "text")
        ):
            return self._show_report(
                model=invoice_sudo,
                report_type=report_type,
                report_ref="oacc_account.account_invoices_oacc",
                download=download,
            )

        else:
            return super().portal_my_invoice_detail(
                invoice_id, access_token, report_type, download, **kw
            )

    @http.route(
        ["/my/invoices/<int:invoice_id>/post"], type="http", auth="user", website=True
    )
    def portal_my_invoice_post(self, invoice_id, from_list=None, **kw):
        """
        Post an invoice from portal
        """
        invoice = request.env["account.move"].browse([invoice_id])
        invoice_sudo = invoice.with_user(SUPERUSER_ID).exists()
        if request.env.user.commercial_partner_id not in (
            invoice_sudo.company_id.partner_id.commercial_partner_id,
            invoice_sudo.acc_operation_id.admin_ids.commercial_partner_id,
        ):
            raise NotFound()

        return_url = (
            "/operation/%d/%s" % (invoice_sudo.acc_operation_id.id, from_list)
            if from_list
            else invoice_sudo.get_portal_url()
        )

        # Valide la facture si en brouillon
        if invoice_sudo.state == "draft":
            invoice_sudo.action_post()
            invoice._message_log(body=_("Facture validée depuis le portail"))
        return request.redirect(return_url)

    @http.route(
        [
            '/operation/<model("acc.operation"):operation>/post_all_draft_invoices',
        ],
        type="http",
        auth="user",
        website=True,
    )
    def portal_all_draft_invoices_post(
        self, operation, filterby=None, search=None, search_in="all"
    ):
        """
        Post all draft invoices from portal
        """
        # Manage search input
        domain = [
            ("state", "=", "draft"),
        ]

        invoices = self._prepare_invoice_portal_values(operation, domain)["invoices"]
        if (
            all(
                invoice.company_id.partner_id.commercial_partner_id
                == request.env.user.commercial_partner_id
                for invoice in invoices
            )
            or request.env.user.commercial_partner_id
            in operation.admin_ids.commercial_partner_id
        ):
            for invoice in invoices:
                invoice._message_log(
                    body=_("Facture validée depuis le portail"),
                    author_id=request.env.user.partner_id.id,
                )
            invoices.action_post()
        else:
            invoices_to_post = invoices.filtered(
                lambda i: i.company_id.partner_id.commercial_partner_id
                == request.env.user.commercial_partner_id
            )
            for invoice in invoices_to_post:
                invoice._message_log(
                    body=_("Facture validée depuis le portail"),
                    author_id=request.env.user.partner_id.id,
                )
            invoices_to_post.action_post()

        return request.redirect_query(
            "/operation/%s/invoices" % operation.id, query=request.params
        )

    @http.route(
        ["/my/invoices/<int:invoice_id>/pay"], type="http", auth="user", website=True
    )
    def portal_my_invoice_pay(self, invoice_id, from_list=None, **kw):
        """
        Mark an invoice as paid from portal
        """
        invoice = request.env["account.move"].browse([invoice_id])
        invoice_sudo = invoice.with_user(SUPERUSER_ID).exists()

        return_url = "/operation/%d/%s" % (invoice_sudo.acc_operation_id.id, from_list)

        if (
            request.env.user.commercial_partner_id
            in (
                invoice_sudo.company_id.partner_id.commercial_partner_id,
                invoice_sudo.acc_operation_id.admin_ids.commercial_partner_id,
            )
            and invoice_sudo.state == "posted"
            and invoice.payment_state in ("not_paid", "partial")
            and invoice.amount_total
            and invoice.move_type == "out_invoice"
        ):
            invoice._message_log(
                body=_("Facture marquée comme payée depuis le portail")
            )
            # Appelle le wizard de paiement
            apr = (
                request.env["account.payment.register"]
                .with_user(SUPERUSER_ID)
                .with_context(
                    active_model="account.move",
                    active_ids=[invoice_id],
                    dont_redirect_to_payments=True,
                )
                .create({})
            )
            # Confirme le paiement
            apr.action_create_payments()

        return request.redirect(return_url)

    @http.route(
        [
            '/operation/<model("acc.operation"):operation>/invoices',
        ],
        type="http",
        auth="user",
        website=True,
    )
    def invoice_list(self, operation):
        """
        List all invoices
        """
        render_values = {}
        render_values.update(self._get_role(operation))

        order_invoices = "state, name desc, start_date desc, partner_id"
        order_contracts = "date_app desc, name desc, seller_id"

        render_values["invoices"] = self._prepare_invoice_portal_values(
            operation, [], order=order_invoices
        )["invoices"]

        render_values["contracts"] = self._prepare_invoice_document_portal_values(
            operation,
            order_contracts,
        )["contracts"]

        render_values.update(
            {
                "default_url": "/operation/" + slug(operation) + "/invoices",
                "page_name": "invoices",
                "page_breadcrumbs": "Factures",
            }
        )

        return request.render("oacc_portal_account.operation_invoice", render_values)
